/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/
package org.gvsig.raster.tools.algorithm.maskthreshold;

import java.util.HashMap;

import javax.swing.SwingUtilities;

import org.gvsig.fmap.dal.coverage.exception.ProcessInterruptedException;
import org.gvsig.fmap.dal.coverage.store.RasterDataStore;
import org.gvsig.i18n.Messages;
import org.gvsig.raster.tools.algorithm.base.RasterBaseAlgorithmLibrary;
import org.gvsig.raster.tools.algorithm.base.process.RasterProcess;

/**
 * Process to create a mask using a threshold. This threshold could be a value
 * of a pixel or a fixed value
 *
 * 10/12/2007
 * @author Nacho Brodin nachobrodin@gmail.com
 */
public class MaskthresholdProcess extends RasterProcess {
	public static String      RASTER_STORE1    = "RasterStore1";
	public static String      RASTER_STORE2    = "RasterStore2";
	public static String      PATH             = "Path";
	public static String      FILENAME         = "FileName";
	public static String      THRESHOLD        = "Threshold";
	public static String      METHOD           = "Method";
	public static String      TIME             = "Time";
	public static String[]    MASK_METHODS     = new String[]{"Greater", "Lower", "GreaterOrEquals", "LoweOrEquals"};
	
	private RasterDataStore   store1           = null;
	private RasterDataStore   store2           = null;
	private String            filename         = null;
	private double            threshold        = 0;
	private int               method           = 0;
	private long              milis            = 0;
	private int               percent          = 0;
	
	public static void registerParameters() {
		RASTER_STORE1 = RasterBaseAlgorithmLibrary.registerInputParameter(RASTER_STORE1, RasterDataStore.class);
		RASTER_STORE2 = RasterBaseAlgorithmLibrary.registerInputParameter(RASTER_STORE2, RasterDataStore.class);
		PATH = RasterBaseAlgorithmLibrary.registerInputParameter(PATH, String.class);
		THRESHOLD = RasterBaseAlgorithmLibrary.registerInputParameter(THRESHOLD, Double.class);
		METHOD = RasterBaseAlgorithmLibrary.registerInputParameter(METHOD, Integer.class);
		
		FILENAME = RasterBaseAlgorithmLibrary.registerOutputParameter(FILENAME, String.class);
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.rastertools.RasterProcess#init()
	 */
	public void init() {
		store1 = (RasterDataStore)getParam(RASTER_STORE1);
		store2 = (RasterDataStore)getParam(RASTER_STORE2);
		filename = getStringParam(PATH);
		method = (Integer)getParam(METHOD);
		threshold = (Double)getParam(THRESHOLD);
	}

	/**
	 * Mtodo donde se ejecutar el Thread, aqu se reproyecta el raster.
	 */
	public void process() throws ProcessInterruptedException {
		long t1 = new java.util.Date().getTime();
		insertLineLog(Messages.getText("maskthreshold"));
		int result = 1;
		try {
			//Algorithm
			if(result == 1)
				throw new MaskthresholdException("");
			
			if(result != 0) {
				if (incrementableTask != null) {
					incrementableTask.processFinalize();
					setProgressActive(false);
				}
				messageBoxError("transformation_not_possible", this);
				return;
			}

			long t2 = new java.util.Date().getTime();
			milis = t2 - t1;
			
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					if (externalActions != null) {
						HashMap<String, Object> map = new HashMap<String, Object>();
						map.put(FILENAME, filename);
						map.put(TIME, new Long(milis));
						externalActions.end(map);
					}
				}
			});
		} catch (MaskthresholdException e) {
			if (incrementableTask != null)
				incrementableTask.processFinalize();
			messageBoxError("error_reprojecting", this, e);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.raster.tools.app.basic.raster.process.RasterProcess#getResult()
	 */
	public Object getResult() {
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put(FILENAME, filename);
		map.put(TIME, new Long(milis));
		return map;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gui.beans.incrementabletask.IIncrementable#getPercent()
	 */
	public int getPercent() {
		return percent;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gui.beans.incrementabletask.IIncrementable#getTitle()
	 */
	public String getTitle() {
		return Messages.getText("maskthreshold");
	}
}